# Create a vector of CH1903LV03 coordinates to be converted to WGS84 coordinates by the use of the swisstopo tool REFRAME

# - Specify the grid in swiss coordinates CH1903LV03 under changable input

# (c) Franziska Gerber, 2018


import math
import numpy as np
import matplotlib.pyplot as plt

#*** Changable input ***
ncols = 191#128               # number of columns
nrows = 190#124               # number of rows
xllcorner = 784475#750920     # x-coordinate of lower left corner
yllcorner = 172200#162000     # y-coordinate of lower left corner
cellsize = 50#450             # grid resolution 
outputfile = "Ascii/SwissCoordinateVector_50m.txt"   # name of output file
#***********************


# Create latitude and longitude vector
latvec = np.linspace(yllcorner, yllcorner+(ncols-1)*cellsize, ncols)
lonvec = np.linspace(xllcorner, xllcorner+(nrows-1)*cellsize, nrows)

# Make latitude and longitude matrix
lonmat = []
for i in range(0,ncols):
    lonmat.append(lonvec)
lonmat = np.asarray(lonmat)
print(np.shape(lonmat))

latmat = []
for ii in range(0,nrows):
    latmat.append(latvec)

latmat = np.asarray(latmat)
latmat = latmat.T
print(np.shape(latmat))


# Concert matrices to vectors
lon1D = np.reshape(lonmat, lonmat.shape[0]*lonmat.shape[1], order = 'C')
lat1D = np.reshape(latmat, latmat.shape[0]*latmat.shape[1], order = 'C')

# Merge vectors
lonlat = (lon1D, lat1D)
lonlat = np.asarray(lonlat)
lonlat = lonlat.T

# Write Ascii-File
header = ""
np.savetxt(outputfile, lonlat, header = header, fmt = "%1.2f")
