# Read WRF nc file and create an arcgis-format ascii output

# - Specify the start and end date and time
# - Choose the variable and output name

# (c) Franziska Gerber, 2018

import matplotlib
# Force matplotlib to not use any Xwindows backend.
matplotlib.use('Agg')
import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.cm import get_cmap
import matplotlib as mpl
from scipy.interpolate import griddata
from wrf import to_np
from read_wrffields import read_wrffields

# Input***********************************
month = 1
DateStart = "2016-01-31"
TimeStart = "03:00:00"
DateEnd = "2016-01-31"
TimeEnd = "06:00:00"
smoothing = "-wrfsmooth4"

Folder = "/scratch/snx3000/gerberf/WRF3.7.1Jan-Mar_smoothtopo1440_cosmo-le-360_precip10_mixing_lambert_1350_d01-4-regulartstep-1350YSU-landsurface-noahmp-highroughness-Z0SNOadapt-varadaptChanghai-ptop150-eta50res"+smoothing+"-Jan/test/em_real/3h/"

Timeindex = 0
domain = "d02"
res = "450"
variable = "ter" # "ter" or "SNOWNC" or "RAINNC"
varout = "hgt" # "hgt" or "snowacc" or "totprecip"

outfiletitle = "ascii-file_wrf_"+domain+"-"+varout+"_"+DateStart+"-"+DateEnd+smoothing+"-5min"
if (res == "300"):
    outfiletitle = "ascii-file_wrf_"+domain+"-"+res+"m-"+varout+"_"+DateStart+"-"+DateEnd+smoothing
#******************************************


# Speciy domains and global coordinate files
if (domain == "d02"):
    globaCoords = "Ascii/GlobalCoordinateVector_450m_changelatlon.txt"
    ncols = 128
    nrows = 124
    xllcorner = 750920
    yllcorner = 162000
    cellsize = 450
    res = 450
if (domain == "d03"):
    globaCoords = "Ascii/GlobalCoordinateVector_300m_changelatlon.txt"
    ncols = 80
    nrows = 70
    xllcorner = 772000
    yllcorner = 171000
    cellsize = 300
    res = 300
if (domain == "d04"):
    globaCoords = "Ascii/GlobalCoordinateVector_50m_changelatlon.txt"
    ncols = 191
    nrows = 190
    xllcorner = 784475
    yllcorner = 172200
    cellsize = 50
    res = 50

# Read wrf field
t_F1, var_F1, hgt_F1, ter_F1, lats_F1, lons_F1 = read_wrffields(Folder, domain, "2016", variable, varadd="")

# Find indices of times
starttime = np.array(DateStart+"T"+TimeStart+".000000000",dtype='a29')
endtime = np.array(DateEnd+"T"+TimeEnd+".000000000",dtype='a29')

timeseries = t_F1.tolist()
dim = np.shape(t_F1)
times = []

for ii in range(dim[0]):
    jj  = int(ii)
    times.append(t_F1[jj])
del t_F1
t_F1 = times

indstart = np.where(t_F1[:] == starttime)
indend = np.where(t_F1[:] == endtime)
indstart = int(indstart[0])
indend = int(indend[0])

# Calculate precipitation during the specified time period
if (variable == "SNOWNC"):
    helpvar = var_F1
    var_F1 = var_F1[indend,:,:]-var_F1[indstart,:,:]

if (variable == "RAINNC"):
    helpvar = var_F1
    var_F1 = var_F1[indend,:,:]-var_F1[indstart,:,:]


#Interpolate on Radar Domain (coordinates - CH1903LV03)

# Read coordinates vector
if (domain == "d04"):
    lonrad, latrad= np.loadtxt(globaCoords, delimiter=' ', usecols=(0, 1), unpack=True)
if (domain == "d03"):
    if (res == 300):
        lonrad, latrad= np.loadtxt(globaCoords, delimiter=' ', usecols=(0, 1), unpack=True)
if (domain == "d02"):
    lonrad, latrad= np.loadtxt(globaCoords, delimiter=' ', usecols=(0, 1), unpack=True)

lon2D = np.reshape(lonrad, (ncols,nrows), order = 'C')    # Vector to matrix
lat2D = np.reshape(latrad, (ncols,nrows), order = 'C')

lonCH = np.linspace(xllcorner, xllcorner+(ncols-1)*cellsize, ncols) # Create swiss coordinate vectors
latCH = np.linspace(yllcorner, yllcorner+(nrows-1)*cellsize, nrows)

lonmat = []                                                # Swiss coordinates to matrix
for i in range(0,nrows):
    lonmat.append(lonCH)
lonmat = np.asarray(lonmat)
lonmat = lonmat.T


latmat = []
for ii in range(0,ncols):
    latmat.append(latCH)

latmat = np.asarray(latmat)
latmat = latmat

# to numpy arrays
nplats = to_np(lats_F1)
nplons = to_np(lons_F1)
npvar = to_np(var_F1)
if (varout == "hgt"):
    npvar = to_np(ter_F1)

# Regrid WRF data on global coordinates corresponding to swiss coordinates
matgrid = griddata((nplons.ravel(), nplats.ravel()), npvar.ravel(), (lon2D, lat2D), method = 'linear')

# Create output header and matrix
outmatrix = np.fliplr(matgrid)
outmatrix = outmatrix.T

# Write Ascii file
header = "ncols "+str(ncols)+"\nnrows "+str(nrows)+"\nxllcorner "+str(xllcorner)+"\nyllcorner "+str(yllcorner)+"\ncellsize "+str(cellsize)+"\nNODATA_value -9999.000"

np.savetxt("Ascii/"+outfiletitle+".txt", outmatrix, header = header, delimiter=' ', fmt = "%1.2f")
